"""Protocol constants used by Django channels."""

RUN_ANALYSIS_CHANNEL = 'lactolyse.runanalysis'

LACTATE_REPORT_TYPE = 'lactolyse.lactate_report'
LACTATE_REPORT_RUN_TYPE = 'lactolyse.lactate_run_report'
CRITICAL_POWER_REPORT_TYPE = 'lactolyse.critical_power_report'
CONCONI_REPORT_TYPE = 'lactolyse.conconi_report'

GROUP_SESSIONS = 'lactolyse.session.{websocket_id}'



# Generated by Django 2.2.24 on 2021-09-05 16:57

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import lactolyse.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('lactolyse', '0006_lactate_run_analysis'),
    ]

    operations = [
        migrations.CreateModel(
            name='CriticalPowerAnalyses',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('created', models.DateField(auto_now_add=True)),
                (
                    'report',
                    models.FileField(
                        upload_to=lactolyse.models.generate_lactate_threshold_name
                    ),
                ),
                ('result', models.IntegerField(null=True)),
                (
                    'athlete',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='lactolyse.Athlete',
                    ),
                ),
                (
                    'contributor',
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='CriticalPowerMeasurement',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('time', models.IntegerField()),
                ('power', models.IntegerField()),
                (
                    'analyses',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='lactolyse.CriticalPowerAnalyses',
                    ),
                ),
            ],
        ),
    ]



"""Conconi test."""
import numpy as np

from .base import BaseAnalysis


class CriticalPowerAnalyses(BaseAnalysis):

    name = 'critical_power'
    template = 'critical_power.tex'

    def _calculate_cp_context(self, time, power):
        inv_time = list(map(lambda t: 1 / t, time))

        power_poly = np.poly1d(np.polyfit(inv_time, power, 1))

        inverse_power_poly = np.poly1d(np.polyfit(power, inv_time, 1))

        work = np.multiply(time, power)
        work_poly = np.poly1d(np.polyfit(time, work, 1))

        return {
            'power': power_poly(0),
            'power_poly': power_poly,
            'inverse_power': inverse_power_poly.roots[0],
            'inverse_power_poly': inverse_power_poly,
            'work': work_poly(0),
        }

    def _calculate_zones(self, power, work):
        poly = lambda time: power + (work / time)
        return {
            '1': power * 0.45,
            '2': power * 0.7,
            '2a': power * 0.77,
            '3': power * 0.95,
            '4': poly(45 * 60),
            '5': poly(10 * 60),
            '6': poly(3 * 60),
        }

    def render_context(self, inputs):
        """Render the context."""
        for attr in ['time', 'power']:
            if attr not in inputs:
                raise ValueError("Missing input '{}'.".format(attr))

        critical_power = self._calculate_cp_context(inputs['time'], inputs['power'])

        return {
            'inputs': inputs,
            'cp': critical_power,
            'zones': self._calculate_zones(
                critical_power['power'],
                critical_power['work'],
            ),
        }

    def get_results(self, context):
        """Return the result of the analysis."""
        return {'result': context['cp']['power']}
